@@ -29,6 +29,7 @@ use arrow::array::*;
 use arrow::compute::cast;
 use arrow::datatypes::*;
 use arrow::util::bench_util::*;
+use arrow::util::decimal::Decimal256;
 use arrow::util::test_util::seedable_rng;
 
 fn build_array<T: ArrowPrimitiveType>(size: usize) -> ArrayRef
@@ -81,6 +82,28 @@ fn build_utf8_date_time_array(size: usize, with_nulls: bool) -> ArrayRef {
     Arc::new(builder.finish())
 }
 
+fn build_decimal128_array(size: usize, precision: usize, scale: usize) -> ArrayRef {
+    let mut rng = seedable_rng();
+    let mut builder = Decimal128Builder::new(size, precision, scale);
+
+    for _ in 0..size {
+        let _ = builder.append_value(rng.gen_range::<i128, _>(0..1000000000));
+    }
+    Arc::new(builder.finish())
+}
+
+fn build_decimal256_array(size: usize, precision: usize, scale: usize) -> ArrayRef {
+    let mut rng = seedable_rng();
+    let mut builder = Decimal256Builder::new(size, precision, scale);
+    let mut bytes = [0; 32];
+    for _ in 0..size {
+        let num = rng.gen_range::<i128, _>(0..1000000000);
+        bytes[0..16].clone_from_slice(&num.to_le_bytes());
+        let _ = builder.append_value(&Decimal256::new(precision, scale, &bytes));
+    }
+    Arc::new(builder.finish())
+}
+
 // cast array from specified primitive array type to desired data type
 fn cast_array(array: &ArrayRef, to_type: DataType) {
     criterion::black_box(cast(array, &to_type).unwrap());
@@ -102,6 +125,9 @@ fn add_benchmark(c: &mut Criterion) {
     let utf8_date_array = build_utf8_date_array(512, true);
     let utf8_date_time_array = build_utf8_date_time_array(512, true);
 
+    let decimal128_array = build_decimal128_array(512, 10, 3);
+    let decimal256_array = build_decimal256_array(512, 50, 3);
+
     c.bench_function("cast int32 to int32 512", |b| {
         b.iter(|| cast_array(&i32_array, DataType::Int32))
     });
@@ -179,6 +205,19 @@ fn add_benchmark(c: &mut Criterion) {
     c.bench_function("cast utf8 to date64 512", |b| {
         b.iter(|| cast_array(&utf8_date_time_array, DataType::Date64))
     });
+
+    c.bench_function("cast decimal128 to decimal128 512", |b| {
+        b.iter(|| cast_array(&decimal128_array, DataType::Decimal128(30, 5)))
+    });
+    c.bench_function("cast decimal128 to decimal256 512", |b| {
+        b.iter(|| cast_array(&decimal128_array, DataType::Decimal256(50, 5)))
+    });
+    c.bench_function("cast decimal256 to decimal128 512", |b| {
+        b.iter(|| cast_array(&decimal256_array, DataType::Decimal128(38, 2)))
+    });
+    c.bench_function("cast decimal256 to decimal256 512", |b| {
+        b.iter(|| cast_array(&decimal256_array, DataType::Decimal256(50, 5)))
+    });
 }
 
 criterion_group!(benches, add_benchmark);
