@@ -15,19 +15,20 @@
 // specific language governing permissions and limitations
 // under the License.
 
-use num::BigInt;
 use std::any::Any;
 use std::sync::Arc;
 
-use crate::array::array_decimal::{BasicDecimalArray, Decimal256Array};
+use crate::array::array_decimal::Decimal256Array;
 use crate::array::ArrayRef;
 use crate::array::Decimal128Array;
 use crate::array::{ArrayBuilder, FixedSizeBinaryBuilder};
 
 use crate::error::{ArrowError, Result};
 
-use crate::datatypes::{validate_decimal256_precision, validate_decimal_precision};
-use crate::util::decimal::{BasicDecimal, Decimal256};
+use crate::datatypes::{
+    validate_decimal256_precision_with_lt_bytes, validate_decimal_precision,
+};
+use crate::util::decimal::Decimal256;
 
 /// Array Builder for [`Decimal128Array`]
 ///
@@ -84,33 +85,14 @@ impl Decimal128Builder {
     /// Appends a decimal value into the builder.
     #[inline]
     pub fn append_value(&mut self, value: impl Into<i128>) -> Result<()> {
-        let value = if self.value_validation {
-            validate_decimal_precision(value.into(), self.precision)?
-        } else {
-            value.into()
-        };
-
-        let value_as_bytes =
-            Self::from_i128_to_fixed_size_bytes(value, Self::BYTE_LENGTH as usize)?;
-        if Self::BYTE_LENGTH != value_as_bytes.len() as i32 {
-            return Err(ArrowError::InvalidArgumentError(
-                "Byte slice does not have the same length as Decimal128Builder value lengths".to_string()
-            ));
+        let value = value.into();
+        if self.value_validation {
+            validate_decimal_precision(value, self.precision)?
         }
+        let value_as_bytes: [u8; 16] = value.to_le_bytes();
         self.builder.append_value(value_as_bytes.as_slice())
     }
 
-    pub(crate) fn from_i128_to_fixed_size_bytes(v: i128, size: usize) -> Result<Vec<u8>> {
-        if size > 16 {
-            return Err(ArrowError::InvalidArgumentError(
-                "Decimal128Builder only supports values up to 16 bytes.".to_string(),
-            ));
-        }
-        let res = v.to_le_bytes();
-        let start_byte = 16 - size;
-        Ok(res[start_byte..16].to_vec())
-    }
-
     /// Append a null value to the array.
     #[inline]
     pub fn append_null(&mut self) {
@@ -201,9 +183,7 @@ impl Decimal256Builder {
     pub fn append_value(&mut self, value: &Decimal256) -> Result<()> {
         let value = if self.value_validation {
             let raw_bytes = value.raw_value();
-            let integer = BigInt::from_signed_bytes_le(raw_bytes);
-            let value_str = integer.to_string();
-            validate_decimal256_precision(&value_str, self.precision)?;
+            validate_decimal256_precision_with_lt_bytes(raw_bytes, self.precision)?;
             value
         } else {
             value
@@ -256,9 +236,9 @@ impl Decimal256Builder {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use num::Num;
+    use num::{BigInt, Num};
 
-    use crate::array::array_decimal::{BasicDecimalArray, Decimal128Array};
+    use crate::array::array_decimal::Decimal128Array;
     use crate::array::{array_decimal, Array};
     use crate::datatypes::DataType;
     use crate::util::decimal::{Decimal128, Decimal256};
@@ -305,21 +285,21 @@ mod tests {
     fn test_decimal256_builder() {
         let mut builder = Decimal256Builder::new(30, 40, 6);
 
-        let mut bytes = vec![0; 32];
+        let mut bytes = [0_u8; 32];
         bytes[0..16].clone_from_slice(&8_887_000_000_i128.to_le_bytes());
-        let value = Decimal256::try_new_from_bytes(40, 6, bytes.as_slice()).unwrap();
+        let value = Decimal256::try_new_from_bytes(40, 6, &bytes).unwrap();
         builder.append_value(&value).unwrap();
 
         builder.append_null();
 
-        bytes = vec![255; 32];
-        let value = Decimal256::try_new_from_bytes(40, 6, bytes.as_slice()).unwrap();
+        bytes = [255; 32];
+        let value = Decimal256::try_new_from_bytes(40, 6, &bytes).unwrap();
         builder.append_value(&value).unwrap();
 
-        bytes = vec![0; 32];
+        bytes = [0; 32];
         bytes[0..16].clone_from_slice(&0_i128.to_le_bytes());
         bytes[15] = 128;
-        let value = Decimal256::try_new_from_bytes(40, 6, bytes.as_slice()).unwrap();
+        let value = Decimal256::try_new_from_bytes(40, 6, &bytes).unwrap();
         builder.append_value(&value).unwrap();
 
         builder.append_option(None::<&Decimal256>).unwrap();
@@ -349,9 +329,9 @@ mod tests {
     fn test_decimal256_builder_unmatched_precision_scale() {
         let mut builder = Decimal256Builder::new(30, 10, 6);
 
-        let mut bytes = vec![0; 32];
+        let mut bytes = [0_u8; 32];
         bytes[0..16].clone_from_slice(&8_887_000_000_i128.to_le_bytes());
-        let value = Decimal256::try_new_from_bytes(40, 6, bytes.as_slice()).unwrap();
+        let value = Decimal256::try_new_from_bytes(40, 6, &bytes).unwrap();
         builder.append_value(&value).unwrap();
     }
 
@@ -385,7 +365,7 @@ mod tests {
             .expect("should not validate invalid value at builder");
 
         let array = builder.finish();
-        let array_data = array_decimal::BasicDecimalArray::data(&array);
+        let array_data = array_decimal::DecimalArray::data(&array);
         array_data.validate_values().unwrap();
     }
 }
