@@ -17,7 +17,6 @@
 
 use std::{io::Read, sync::Arc};
 
-use byteorder::{ByteOrder, LittleEndian};
 use parquet_format::{ColumnOrder as TColumnOrder, FileMetaData as TFileMetaData};
 use thrift::protocol::TCompactInputProtocol;
 
@@ -62,19 +61,8 @@ pub fn parse_metadata<R: ChunkReader>(chunk_reader: &R) -> Result<ParquetMetaDat
         ));
     }
 
-    let mut metadata = Vec::with_capacity(metadata_len);
-
-    let read = chunk_reader
-        .get_read(file_size - footer_metadata_len as u64, metadata_len)?
-        .read_to_end(&mut metadata)?;
-
-    if read != metadata_len {
-        return Err(eof_err!(
-            "Expected to read {} bytes of metadata, got {}",
-            metadata_len,
-            read
-        ));
-    }
+    let metadata =
+        chunk_reader.get_bytes(file_size - footer_metadata_len as u64, metadata_len)?;
 
     decode_metadata(&metadata)
 }
@@ -112,7 +100,7 @@ pub fn decode_footer(slice: &[u8; FOOTER_SIZE]) -> Result<usize> {
     }
 
     // get the metadata length from the footer
-    let metadata_len = LittleEndian::read_i32(&slice[..4]);
+    let metadata_len = i32::from_le_bytes(slice[..4].try_into().unwrap());
     metadata_len.try_into().map_err(|_| {
         general_err!(
             "Invalid Parquet file. Metadata length is less than zero ({})",
